// compose2.cpp: Abstracts the function argument type
#include <iostream>
#include <numeric>
#include <vector>
using namespace std;

template<class T>
class Composer {
private:
    typedef T (*Fun)(T);
    const vector<Fun>& funs;
public:
    Composer(const vector<Fun>& fs) : funs(fs) {}
    T operator()(T x) const {
        T result = x;
        for (typename vector<Fun>::const_reverse_iterator p = funs.rbegin(); p != funs.rend(); ++p)
            result = (*p)(result);
        return result;
    }
};

double f(double x) {
    return x*x;
}

double g(double x) {
    return x+1;
}

double h(double x) {
    return x/3.0;
}

string f2(string s) { return s + "s"; }
string g2(string s) { return "the" + s; }
string h2(string s) { return " " + s; }

int main() {
    typedef double (*Fun)(double);
    vector<Fun> funs;
    funs.push_back(h);
    funs.push_back(f);
    funs.push_back(g);
    Composer<double> comp(funs);
    cout << comp(2.0) << endl;  // 3

    typedef string (*Fun2)(string);
    vector<Fun2> funs2;
    funs2.push_back(f2);
    funs2.push_back(g2);
    funs2.push_back(h2);
    Composer<string> comp2(funs2);
    cout << comp2("boy") << endl; // "the boys"
}
