// grid.cpp: Stores an x-y graph in a grid of chars
#include <algorithm>
#include <iostream>
#include <iterator>
#include <vector>
using namespace std;

namespace
{
   typedef vector<char> row_type;
   typedef vector<row_type> grid_type;
   const int xmax = 15;    	// # of columns
   const int ymax = xmax;  	// # of rows

   inline double f(double x) {
      // NOTE: Contrived to fix within grid!
      return x * x / ymax + 1.0;
   }
   inline void print_row(const row_type& row) {
      copy(row.begin(), row.end(),ostream_iterator<char>(cout, ""));
      cout << endl;
   }
}

int main() {
    // Draw y-axis and clear 1st quadrant
    row_type blank_row;
    fill_n(back_inserter(blank_row), xmax, ' ');
    blank_row[0] = '|';
    grid_type grid;		      // A vector of rows
    fill_n(back_inserter(grid), ymax, blank_row);

    // Draw x-axis:
    grid[0][0] = '+';
    fill(grid[0].begin()+1, grid[0].end(), '-');

    // Populate with points (x, f(x))
    for (int x = 0; x < xmax; ++x)
        grid[f(x)][x] = '*';	// Remember: it's row-oriented

    for_each(grid.rbegin(), grid.rend(), print_row);
}

